@@ -249,21 +249,27 @@ namespace FCam { namespace N900 {
             }
             current.gain = req->gain = v4l2Sensor->getGain();
 
-            if (req->shot().whiteBalance != current._shot.whiteBalance) {
-                int wb = req->shot().whiteBalance;
-
-                // Very lenient sanity checks - restricting the range is up to the auto-wb algorithm
-                if (wb < 0) wb = 0; // a black-body radiator at absolute zero.
-                if (wb > 25000) wb = 25000; // A type 'O' star.
-
-                float matrix[12];
-                sensor->rawToRGBColorMatrix(wb, matrix);
-                v4l2Sensor->setWhiteBalance(matrix);
-
-                current._shot.whiteBalance = req->shot().whiteBalance;
-                current.whiteBalance = wb;            
+            if (req->shot().colorMatrix().size() != 12) {
+                if (req->shot().whiteBalance != current._shot.whiteBalance) {                
+                    int wb = req->shot().whiteBalance;
+                    
+                    // Very lenient sanity checks - restricting the range is up to the auto-wb algorithm
+                    if (wb < 0) wb = 0; // a black-body radiator at absolute zero.
+                    if (wb > 25000) wb = 25000; // A type 'O' star.
+                    
+                    float matrix[12];
+                    sensor->platform().rawToRGBColorMatrix(wb, matrix);
+                    v4l2Sensor->setWhiteBalance(matrix);
+                    
+                    current._shot.whiteBalance = req->shot().whiteBalance;
+                    current.whiteBalance = wb;            
+                }
+                req->whiteBalance = current.whiteBalance;
+            } else {
+                // custom WB - always set it                
+                v4l2Sensor->setWhiteBalance(&req->shot().colorMatrix()[0]);
+                current._shot.setColorMatrix(&req->shot().colorMatrix()[0]);
             }
-            req->whiteBalance = current.whiteBalance;
 
             // Predict when this frame will be done. It should be HS_VS +
             // the readout time for the previous frame + the frame time
@@ -385,7 +391,8 @@ namespace FCam { namespace N900 {
             V4L2Sensor::Mode m;
             m.width  = req->shot().image.width();
             m.height = req->shot().image.height();
-            m.type   = req->shot().image.type();
+            // enforce UYVY or RAW
+            m.type   = req->shot().image.type() == RAW ? RAW : UYVY;
             v4l2Sensor->startStreaming(m, req->shot().histogram, req->shot().sharpness);
             v4l2Sensor->setFrameTime(0);
         
@@ -521,6 +528,9 @@ namespace FCam { namespace N900 {
                 dprintf(3, "Handler: Expected a frame at %d %d, but one didn't arrive until %d %d\n",
                         req->processingDoneTime.s(), req->processingDoneTime.us(),
                         f->processingDoneTime.s(), f->processingDoneTime.us());
+                error(Event::ImageDroppedError, sensor,
+                      "Expected image data not returned from V4L2. Likely caused by excess"
+                      " page faults (thrashing).");
                 req->image = Image(req->image.size(), req->image.type(), Image::Discard);
                 if (!req->shot().wanted) {
                     delete req;
@@ -556,11 +566,17 @@ namespace FCam { namespace N900 {
                             error(Event::ResolutionMismatch, sensor, 
                                   "Requested image size (%d x %d) "
                                   "on an already allocated image does not "
-                                  "match actual image size (%d x %d). Dropping image data.\n",
+                                  "match actual image size (%d x %d). Dropping image data.",
                                   req->shot().image.width(), req->shot().image.height(),
                                   req->image.width(), req->image.height());
                             req->image = Image(req->image.size(), req->image.type(), Image::Discard);
                             // TODO: crop instead?
+                        } else if (req->image.type() != req->shot().image.type()) {
+                            error(Event::FormatMismatch, sensor, 
+                                  "Requested unsupported image format %d "
+                                  "for an already allocated image.",
+                                  req->shot().image.type());
+                            req->image = Image(req->image.size(), req->image.type(), Image::Discard);
                         } else { // the size matches
                             req->image = req->shot().image;
                             // figure out how long I can afford to wait
